@@ -6,6 +6,7 @@ require 'uri'
 
 require 'recaptcha/configuration'
 require 'recaptcha/helpers'
+require 'recaptcha/reply'
 require 'recaptcha/adapters/controller_methods'
 require 'recaptcha/adapters/view_methods'
 if defined?(Rails)
@@ -55,7 +56,7 @@ module Recaptcha
   end
 
   def self.invalid_response?(resp)
-    resp.empty? || resp.length > configuration.response_limit
+    resp.empty? || resp.length > configuration.response_limit || resp.length < configuration.response_minimum
   end
 
   def self.verify_via_api_call(response, options)
@@ -76,21 +77,10 @@ module Recaptcha
     body['event']['expectedAction'] = options[:action] if options.key?(:action)
     body['event']['userIpAddress'] = options[:remote_ip] if options.key?(:remote_ip)
 
-    reply = api_verification_enterprise(query_params, body, project_id, timeout: options[:timeout])
-    score = reply.dig('riskAnalysis', 'score')
-    token_properties = reply['tokenProperties']
-    success = !token_properties.nil? &&
-      token_properties['valid'].to_s == 'true' &&
-      hostname_valid?(token_properties['hostname'], options[:hostname]) &&
-      action_valid?(token_properties['action'], options[:action]) &&
-      score_above_threshold?(score, options[:minimum_score]) &&
-      score_below_threshold?(score, options[:maximum_score])
-
-    if options[:with_reply] == true
-      [success, reply]
-    else
-      success
-    end
+    raw_reply = api_verification_enterprise(query_params, body, project_id, timeout: options[:timeout])
+    reply = Reply.new(raw_reply, enterprise: true)
+    result = reply.success?(options)
+    options[:with_reply] == true ? [result, reply] : result
   end
 
   def self.verify_via_api_call_free(response, options)
@@ -98,43 +88,10 @@ module Recaptcha
     verify_hash = { 'secret' => secret_key, 'response' => response }
     verify_hash['remoteip'] = options[:remote_ip] if options.key?(:remote_ip)
 
-    reply = api_verification_free(verify_hash, timeout: options[:timeout])
-    success = reply['success'].to_s == 'true' &&
-      hostname_valid?(reply['hostname'], options[:hostname]) &&
-      action_valid?(reply['action'], options[:action]) &&
-      score_above_threshold?(reply['score'], options[:minimum_score]) &&
-      score_below_threshold?(reply['score'], options[:maximum_score])
-
-    if options[:with_reply] == true
-      [success, reply]
-    else
-      success
-    end
-  end
-
-  def self.hostname_valid?(hostname, validation)
-    validation ||= configuration.hostname
-
-    case validation
-    when nil, FalseClass then true
-    when String then validation == hostname
-    else validation.call(hostname)
-    end
-  end
-
-  def self.action_valid?(action, expected_action)
-    case expected_action
-    when nil, FalseClass then true
-    else action == expected_action
-    end
-  end
-
-  def self.score_above_threshold?(score, minimum_score)
-    !minimum_score || (score && score >= minimum_score)
-  end
-
-  def self.score_below_threshold?(score, maximum_score)
-    !maximum_score || (score && score <= maximum_score)
+    raw_reply = api_verification_free(verify_hash, timeout: options[:timeout], json: options[:json])
+    reply = Reply.new(raw_reply, enterprise: false)
+    result = reply.success?(options)
+    options[:with_reply] == true ? [result, reply] : result
   end
 
   def self.http_client_for(uri:, timeout: nil)
@@ -152,11 +109,18 @@ module Recaptcha
     instance
   end
 
-  def self.api_verification_free(verify_hash, timeout: nil)
-    query = URI.encode_www_form(verify_hash)
-    uri = URI.parse("#{configuration.verify_url}?#{query}")
+  def self.api_verification_free(verify_hash, timeout: nil, json: false)
+    if json
+      uri = URI.parse(configuration.verify_url)
+      request = Net::HTTP::Post.new(uri.request_uri)
+      request['Content-Type'] = 'application/json; charset=utf-8'
+      request.body = JSON.generate(verify_hash)
+    else
+      query = URI.encode_www_form(verify_hash)
+      uri = URI.parse("#{configuration.verify_url}?#{query}")
+      request = Net::HTTP::Get.new(uri.request_uri)
+    end
     http_instance = http_client_for(uri: uri, timeout: timeout)
-    request = Net::HTTP::Get.new(uri.request_uri)
     JSON.parse(http_instance.request(request).body)
   end
 
@@ -165,6 +129,7 @@ module Recaptcha
     uri = URI.parse("#{configuration.verify_url}/#{project_id}/assessments?#{query}")
     http_instance = http_client_for(uri: uri, timeout: timeout)
     request = Net::HTTP::Post.new(uri.request_uri)
+    request['Referer'] = Rails.application.default_url_options[:host] if defined? Rails.application
     request['Content-Type'] = 'application/json; charset=utf-8'
     request.body = JSON.generate(body)
     JSON.parse(http_instance.request(request).body)
